"""\
$id$

This file is an installation script for this product.
It is meant to be used as an External Method.

To use, either use Plone 2.0+ quickinstaller to install, or add an external
method to the root of the Plone Site that you want the skin registered
in with the configuration:

 id: install
 title: Install Skin *optional*
 module name: PlonePortlets.Install
 function name: install

Then go to the management screen for the newly added external method
and click the 'Try it' tab.
"""

from cStringIO import StringIO

from Products.CMFCore.utils import getToolByName

from Products.Archetypes.public import listTypes
from Products.Archetypes.Extensions.utils import installTypes
from Products.Archetypes.Extensions.utils import install_subskin

from Products.PlonePortlets.config import *
from Products.PlonePortlets import portlets
from Products.PlonePortlets import permissions
from Products.PlonePortlets import PortletsTool

from Products.PlonePortlets.Extensions.utils import registerPortlets
from Products.PlonePortlets.Extensions.utils import initializePortlet

skin_names = ('PlonePortlets', 'portlet_contents', 'PlonePortletPatches')

configlets = \
( { 'id'         : 'PlonePortlets'
  , 'name'       : 'Assign Portlets'
  , 'action'     : 'string:${portal_url}/prefs_portlets'
  , 'category'   : 'Products'
  , 'appId'      : 'PlonePortlets'
  , 'permission' : permissions.MANAGE_PORTAL_PERMISSION
  , 'imageUrl'   : 'skins_icon.gif'
  }
,
  { 'id'         : 'PlonePortletsAdd'
  , 'name'       : 'Add/Remove Portlets'
  , 'action'     : 'string:${portal_url}/portal_portlets'
  , 'category'   : 'Products'
  , 'appId'      : 'PlonePortlets'
  , 'permission' : permissions.MANAGE_PORTAL_PERMISSION
  , 'imageUrl'   : 'skins_icon.gif'
  }
,
  { 'id'         : 'PloneGroupPortlets'
  , 'name'       : 'Assign Portlets to groups'
  , 'action'     : 'string:${portal_url}/prefs_portlets_selectgroup'
  , 'category'   : 'Products'
  , 'appId'      : 'PlonePortlets'
  , 'permission' : permissions.MANAGE_PORTAL_PERMISSION
  , 'imageUrl'   : 'skins_icon.gif'
  }
,
  { 'id'         : 'MemberPortlets'
  , 'appId'      : 'PlonePortlets'
  , 'name'       : 'Personal Portlet Preferences'
  , 'action'     : 'string:${portal_url}/personalize_portlets'
  , 'category'   : 'Member'
  , 'permission' : permissions.SET_OWN_PROPERTIES_PERMISSION
  , 'imageUrl'   : 'skins_icon.gif'
  }
)

def setupTool(self, out):
    # If there already is a portlets tool we leave it to avoid deleting existing portlet configurations
    if getToolByName(self, 'portal_portlets', None) is None:
        addTool = self.manage_addProduct['PlonePortlets'].manage_addTool
        addTool('PortletsTool')
        out.write('Adding Portlets Tool\n')

        catalog = getToolByName(self, 'portal_portlets')
        if getattr(catalog.Indexes, 'SearchableText', None) is not None:
            catalog.manage_delIndex(['SearchableText'])
        if getattr(catalog.Indexes, 'Subject', None) is not None:
            catalog.manage_delIndex(['Subject'])
        catalog.manage_addIndex('UID', 'FieldIndex')
        catalog.addColumn('UID')
        out.write('Fixing indexes in Portlets Tool\n')

        typesTool = getToolByName(self, 'portal_types')
        typesTool.manage_addTypeInformation('Factory-based Type Information', id='PortletsTool')
        t = getattr(typesTool, 'PortletsTool')
        t.title = 'PortletsTool'
        t.product = 'PlonePortlets'
        t.content_meta_type = 'PortletsTool'
        t.allowed_content_types = ('Portlet',)
        t.content_icon = 'skins_icon.gif'
        t.addAction('View', 'View', 'string:${object_url}/folder_contents', '',
                    (permissions.VIEW_PERMISSION,), 'object', visible=1)

        wf_tool = getToolByName(self, 'portal_workflow')
        wf_tool.setChainForPortalTypes(('PortletsTool',), '')
        return True

    else:
        out.write('Portlets Tool already existed, skipping...\n')
        return False

def addConfiglets(self, out):
    """Adds the configlets to the Plone Control Panel."""
    configTool = getToolByName(self, 'portal_controlpanel', None)
    if configTool:
        for conf in configlets:
            out.write('Adding configlet %s\n' % conf['id'])
            configTool.registerConfiglet(**conf)


def setupDefaultPortlets(self,out):
    """Instantiates and links the default portlets so that we see it working."""
    portletsTool = getToolByName(self, 'portal_portlets', None)
    if INSTALL_CONFIRMPORTLET:
        initializePortlet(portletsTool, 'StaticPortlet', 'confirmationportlet', 'column2',
                      title='Success', body='PlonePortlets was successfully installed.')
    initializePortlet(portletsTool, 'LoginPortlet', 'login', 'column1')

##def setupPortletCatalogLinks(self,classes):
##    ArchetypesTool = getToolByName(self, 'archetype_tool')
##    for klass in classes:
##        ArchetypesTool.setCatalogsByType(klass['meta_type'], ['portal_portlets','uid_catalog'])


def install(self):
    out = StringIO()
    install_subskin(self, out, GLOBALS)
    first_time = setupTool(self, out)
    classes = listTypes(PROJECTNAME)
    installTypes(self, out, classes, PROJECTNAME)
    #setupPortletCatalogLinks(self,classes)
    registerPortlets(self,classes)
    addConfiglets(self, out)
    if first_time:
        setupDefaultPortlets(self,out)

    out.write('Installation completed.\n')
    return out.getvalue()


#
# Uninstall methods
#

def removeConfiglets(self, out):
    configTool = getToolByName(self, 'portal_controlpanel', None)
    if configTool:
        for conf in configlets:
            out.write('Removing configlet %s\n' % conf['id'])
            configTool.unregisterConfiglet(conf['id'])


def uninstall(self):
    out = StringIO()
    removeConfiglets(self, out)
    out.write('manual uninstall completed.\n')
    return out.getvalue()

