try:
    from Interface import Interface
except ImportError:
    # for Zope versions before 2.6.0
    from Interface import Base as Interface


class IPortlet(Interface):
    """
    A Portlet is a small piece of the visual display, usually containing information,
    links or lists of objects.
    """

    def portletName(self):
        """
        Returns a sanitized name for the portlet for use in templates.
        """

    def getPortletEmitter():
        """
        Returns a string that is the name of the output emitter, typically a
        page template, that will be used for rendering the portlet's contents.
        """

    def getPortletData():
        """
        Returns the portlet contents, as strings, objects, brains, whatever.
        """

    def hasPortletData():
        """
        Returns true if the Portlet has any contents and should be rendered.
        """

    def portletVisibilityInContext(context):
        """
        Returns true/false whether to render the portlet in a particular context.
        """


class IPortletsTool(Interface):
    """
    A tool to handle the Portlets.
    """

##    Methods from PortletsTool class (used while working on syncing interface)
##    getAvailablePortlets
##    getPortletsInContext
##    rewrapPortlet
##    renderPortletGroup
##    addPortletToGroup
##    removePortletFromGroup
##    addPortletToFilter
##    addPersonalPortletToGroup
##    addPersonalFilter
##    setPersonalPortlets
##    getPersonalPortlets
##    getPersonalPortletsGroup

    def getAvailablePortlets():
        """
        Returns all available portlet objectsi.
        """

    def findPortletsForContext(context):
        """
        Returns all UIDs of portlets to render for a category, given the current calling context.
        """

    def getPortletsSection(context, section):
        """
        Gets all portlets in a given section by context.
        """

    def getPortletsSectionExplicit(self, context, section):
        """
        Gets all portlets assigned to a secition in this excplicit context.
        Avoiding acquisition.
        """

    def renderPortletsSection(context, section):
        """
        Renders one section of portlets on screen.
        """

    def getHelperCSS(self, context):
        """
        Returns the names of linked CSS files to include for the portlets
        that are currently being rendered.
        """

    def getHelperJS(self, context):
        """
        Returns the names of linked Javascript files to include for the portlets
        that are currently being rendered.
        """

    def addPortletToSection(context, portletUID, section):
        """
        Appends a portlet to a category for rendering.
        """

    def removePortletFromSection(context, portletUID, section):
        """
        Removes a portlet UID from a category.
        """

    def addPortletToFilter(context, portletUID):
        """
        Adds a portlet to the contextual filter.
        """

    def addPersonalPortletToSection(portletUID, section):
        """
        Adds a portlet UID to a section to render per user.
        """

    def setPersonalPortlets(data):
        """
        Sets the personal portlets for the current user.
        """

    def getPersonalPortlets(self):
        """
        Gets the personal portlets for the current user.
        Returns a dict {section:[uids], etc}.
        """

    def setPersonalPortletsBySection(portletUIDs, section):
        """
        Sets the personal portlets for the current users for a section.
        The portletUIDs is suppoed to be a list of UIDs.
        """

    def getPersonalPortletsBySection(self, section):
        """
        Gets the personal portlets for a user for a section.
        A list of UIDs.
        """

    def getMyGroupPortlets(self):
        """
        Gets the portlets assigned to all groups the current user is
        a member of.
        Returns a dict {section:[uids], etc}.
        """

    def getPortletsForUserGroup(usergroup):
        """
        Gets the portlets assigned to an arbitrary group.
        Returns a dict {section:[uids], etc}.
        """

    def getPortletsForUserGroupBySection(usergroup, section):
        """
        Gets the portlets assigned to an arbitrary group for a specified
        section.
        Returns a list of UIDs.
        """

    def setPortletsForUserGroup(usergroup, data):
        """
        Sets portlets for a defined group of users.
        The data is a dict with {section:[uids], etc}.
        """

    def setPortletsSection(self, portletUIDs, section, context=None, usergroup=None):
        """
        Saves a list of portletUIDs for a defined section.
        For either a group or an acquisition context.
        """

