from Products.PluginIndexes import PluggableIndex
from Products.CMFCore.utils import getToolByName
from Products.Archetypes.public import *
from Products.Archetypes.config import TOOL_NAME
from Products.PlonePortlets.BasePortlet import Portlet
from Products.PlonePortlets.BasePortlet import PortletSchema

ListingSchema = PortletSchema + Schema((
        # XXX choose catalog when we have updating vocabularies
#    StringField('catalog',
#        default='portal_catalog',
#        widget = SelectionWidget(
#            label = 'Choose catalog',
#        ),
#        vocabulary = 'listCatalogs',
#    ),

    LinesField('typestolist',
        widget = MultiSelectionWidget(
        ),
        vocabulary = 'listTypes',
    ),

    StringField('sortindex',
        widget = SelectionWidget(
            label = 'Sort Index',
        ),
        vocabulary = 'getSortableIndexes',
    ),

    StringField('sortorder',
        widget = SelectionWidget(
            label = 'Sort direction',
        ),
        vocabulary = (
            ('', 'Ascending'),
            ('reverse', 'Descending')
        ),
    ),

    LinesField('workflowstates',
        widget = MultiSelectionWidget(
            label = 'Workflow states',
        ),
        vocabulary = 'listWorkflowStates',
    ),

    IntegerField('numberofresults',
                 default = 5,
    ),

))


class ListingPortlet(Portlet):
    """A class for Listing Portlets."""

    schema = ListingSchema
    meta_type = 'ListingPortlet'
    archetype_name = 'Recent Objects Portlet'

    def getPortletEmitter(self):
        """Returns a template to render the contents of the portlet."""
        return 'portlet_default_contents'

    def _fetchPortletData(self):
        """Gets the content."""
        # XXX choose catalog when we have updating vocabularies
#        catalogname = self.getCatalog() or 'portal_catalog'
#        ct=getToolByName(self, catalogname)
        ct = getToolByName(self, 'portal_catalog')
        query = {}
        if self.getSortindex():
            query['sort_on'] = self.getSortindex()
        if self.getSortindex() and self.getSortorder():
            query['sort_order'] = self.getSortorder()
        if self.getSortindex() and self.getNumberofresults():
            query['sort_limit'] = self.getNumberofresults()
        if self.getWorkflowstates():
            query['review_state'] = self.getWorkflowstates()
        if self.getTypestolist():
            query['portal_type'] = self.getTypestolist()

        results = ct(**query)

        if self.getNumberofresults():
            results = results[:self.getNumberofresults()]
##        results = ct.searchResults(sort_on=self.getSortindex(),
##                                   sort_order=self.getSortorder(),
##                                   review_state='published',
##                                   Type=self.getTypestolist(),
##                                   )[:self.getNumberofresults()]
        return results

    def listCatalogs(self):
        """ List the catalogs in the site """
        at = getToolByName(self, TOOL_NAME, None)
        if at is not None:
            return at.getCatalogsInSite()
        else:
            return ['portal_catalog']

    def listTypes(self):
        """XXX"""
        ct = getToolByName(self, 'portal_types')
        types = [(t.getId(), t.title_or_id()) for t in ct.objectValues()]
        types.insert(0, ('', 'All types'))
        return types

    def getWorkflowstatesInCatalog(self):
        """XXX"""
        ct = getToolByName(self, 'portal_catalog')
        states = ct.uniqueValuesFor('review_state')
        return states

    def listWorkflowStates(self):
        """ Return all workflow states """
        wft = getToolByName(self, 'portal_workflow')
        wfstates = {}
        for wf in wft.objectValues():
            for state in wf.states.objectValues():
                wfstates[state.getId()] = state.title_or_id()
        states = wfstates.items()
        states.insert(0, ('', 'No states'))
        return states

    def getCatalogSchema(self):
        """XXX"""
        ct = getToolByName(self, 'portal_catalog')
        return ct.schema()

    def getSortableIndexes(self):
        """Gets indexes for which sorting is possible."""
        ct = getToolByName(self, 'portal_catalog')
        indexes = [(i.getId(), i.getId()) for i in ct.index_objects() if PluggableIndex.SortIndex.isImplementedBy(i)]
        indexes.insert(0, ('', 'No sorting'))
        return indexes


registerType(ListingPortlet)
